-- Fishing Log by David Down
-- coding: utf-8 '�

import "Turbine.Gameplay"
import "Turbine.UI.Lotro"
import "Vinny.Common.EII_ID"
import "Vinny.FishingLog.FL_Data"

function print(text) Turbine.Shell.WriteLine("<rgb=#00FFFF>FL:</rgb> "..tostring(text)) end
function printh(text) print("<rgb=#00FF00>"..text.."</rgb>") end
function printe(text) print("<rgb=#FF6040>Error: "..text.."</rgb>") end

import "Vinny.Common.Help"

local locPat = "You are on %a* server %d* at r(%d) lx(%d+) ly(%d+) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)"
local xlink = "<Examine:IIDDID:0x0000000000000000:0x700%s>[%s]<\\Examine>"
local xpat = "<Examine:IIDDID:0x0%x+:0x700(%x+)>%[(.-)%]<\\Examine>"
local fpPat = "Your proficiency in Fishing has increased to (%d+)."
local Zloc = "^(.+): (.+): (%d+%.%d[NS]), (%d+%.%d[EW])$"
local locStr,locTbl
Track = false
-- You have acquired: [Minnow].
-- Your proficiency in Fishing has increased to 9.

Plugins.FishingLog.Open = function(sender,args)
	FL_window:SetVisible( true )
	FL_window:SetZOrder( 2 )
end

FL_Options = Turbine.PluginData.Load(Turbine.DataScope.Server,"FL_Options")

local FLv = "Fishing Log "..Plugins["FishingLog"]:GetVersion()

Locs = Turbine.PluginData.Load(Turbine.DataScope.Server,"FL_Locs")
if type(Locs) ~= "table" then Locs = {} end
Totals = Turbine.PluginData.Load(Turbine.DataScope.Character,"FL_Totals")
if type(Totals) ~= "table" then 
	Totals = {} 
	print("Created new fishing record")
end

import "Vinny.FishingLog.FL_Window"

function pos(n0,ls,os)
	return (tostring(ls) +math.fmod(tostring(os),20)/20 -n0)/10
end

-- Save player name for later use
player = Turbine.Gameplay.LocalPlayer.GetInstance()
pname = player:GetName()

Turbine.Chat.Received = function (sender,args)
	local msg = args.Message
	local fp = msg:match(fpPat)
	if fp then Totals.fp = fp return end
--	print("Chat type="..args.ChatType)
	if args.ChatType==36 then 
		if msg:find("You have acquired: ") then
			local id,name = msg:match(xpat)
			if not id then id,name = Vinny.Common.EII_ID(msg) end
			if not id then return end
			if ID[id] then
				if not Totals[id] then
					Totals[id] = 0
					print("New type of fish found.")
				end
				Totals[id] = Totals[id]+1
				print("Caught a "..ID[id].n..", count="..Totals[id])
				if locTbl then
					if not locTbl[id] then locTbl[id] = 0 end
					locTbl[id] = locTbl[id]+1
					locTbl.n = locTbl.n+1
				end
			elseif Track then 
				printe("Unknown: "..name..", id="..tostring(id)) 
			end
		end
		return
	end
	if args.ChatType~=Turbine.ChatType.Standard then return end
	local tt = msg:match("^Taking the contents of the (.+)....$")
	if tt then last = last==tt return end
	local r,lx,ly,ox,oy,oz = msg:match(locPat)
	if not r then return end
	local ew,ns = pos(x0,lx,ox), pos(y0,ly,oy)
	locStr = string.format("%.1f,%.1f",ns,ew)
	print("r="..r..", ns="..ns..", ew="..ew.."  ("..locStr..")")
end

local function distance(dy,dx) return math.sqrt(dy*dy+dx*dx) end

local function locV(str,neg)
    local nbr = tonumber(str:sub(1,-2))
    if str:sub(-1)==neg then nbr = -nbr end
    return nbr
end

local function print_list(list)
	local nr,t = 0,{}
	for id,n in pairs(list) do
		if #id==5 then
			table.insert(t,id)
		end
	end
	table.sort(t, function(a,b) return ID[a].n<ID[b].n end )
	for i,id in ipairs(t) do
		local t,n = ID[id],list[id]
		local s = string.format(xlink,id,t.n)
		print(s..': '..n)
		nr = nr + n
	end
	print("Total catch count: "..nr)
	return
end

FL_Command = Turbine.ShellCommand()
function FL_Command:GetShortHelp() return Vinny.Common.Help(help,"??") end
function FL_Command:GetHelp() return Vinny.Common.Help(help,"help") end

printh(FLv..", data loaded.")

function FL_Command:Execute( cmd,args )
	if Vinny.Common.HelpCmd(cmd,args,help) then return end
    if cmd=="fll" then
		if args=="list" then
			printh("Recorded fishing locations:")
			for loc,t in pairs(Locs) do
				print(RegN[t.r]..'('..t.a..'): '..loc..' = '..t.n)
			end
			return
		end
		if args=="last" then
			if locStr then
				printh("Fish caught at "..locStr)
				print_list(locTbl)
			else printe("No location selected yet") end
			return
		end
		local tbl = Locs[args]
		if tbl then
			printh("Fish caught at "..args)
			print_list(tbl)
			return
		end
		local reg,a,y,x = args:match(Zloc)
		if y then
			local r = Region[reg]
			if not r then printe("Unknown region: "..reg) return end
			local y1,x1,ln = locV(y,'S'), locV(x,'W')
			local loc = r..';'..y1..','..x1
			locStr = nil
			for loc,t in pairs(Locs) do
				if t.r==r then
					if distance(y1-t.y,x1-t.x)<1 then locStr = loc break end
				end
			end
			if not locStr then
				locStr = y..' '..x
				Locs[locStr] = {r=r,a=a,y=y1,x=x1,n=0}
				print("Adding new location at "..locStr)
			else print("Found location "..locStr) end
			locTbl = Locs[locStr]
		else printe("Invalid location.") end
		return
	end
	if args=="show" or cmd=="flw" then
		FL_window:SetVisible( true )
		return
	end
    if args=="" then
		local fp = Totals.fp
		if fp then print("Fishing proficiency is "..fp)
		else print("Unknown fishing proficiency.") end
		return
	end
    if args=="catch" then
		printh("Fishing catch record:")
		print_list(Totals)
		return
	end
    if args=="cook" then
		printh("Fish that can be cooked:")
		local t = {}
		for id,tb in pairs(ID) do
			if tb.c=='C' then
				table.insert(t,id)
			end
		end
		table.sort(t, function(a,b) return ID[a].n<ID[b].n end )
		for i,id in ipairs(t) do
			local t = ID[id]
			local s = string.format(xlink,id,t.n)
			print(s..': '..Tier[t.t])
		end
		return
	end
    if args=="trophy" then
		printh("Fish that are trophies:")
		local t = {}
		for id,tb in pairs(ID) do
			if tb.c=='T' then
				table.insert(t,id)
			end
		end
		table.sort(t, function(a,b) return ID[a].n<ID[b].n end )
		for i,id in ipairs(t) do
			local t = ID[id]
			local s = string.format(xlink,id,t.n)
			print(s)
		end
		return
	end
    local id,name = args:match(xpat)
	if not id then id,name = Vinny.Common.EII_ID(args) end
	if id then
		local t = ID[id]
		if t then
			local s = string.format(xlink,id,t.n)
			local c = Cat[t.c]
			if t.t then c = Tier[t.t]..' '..c end
			local v = ''
			if t.v then v = " worth "..t.v.." copper" end
			print(s.." is a "..c.." item"..v)
			local n = Totals[id]
			if n then print("Recorded catches: "..n) end
		else print("Unknown item: "..args..", id="..id) end
		return
	end
    printe("Unknown command, "..args)
end

Turbine.Shell.AddCommand( "fl;fll;flw;fl?", FL_Command )

Plugins.FishingLog.Open = function(sender,args)
	FL_window:SetVisible( true )
	FL_window:SetZOrder( 2 )
end

Plugins.FishingLog.Unload = function(sender,args)
    Turbine.PluginData.Save(Turbine.DataScope.Server,"FL_Locs",Locs)
    Turbine.PluginData.Save(Turbine.DataScope.Character,"FL_Totals",Totals)
    print("Fishing record saved.")
end

--~ if Event[-1] then FR_Command:Execute("fr","") end

-- Options panel
import "Vinny.Common.Options"
OP = Vinny.Common.Options_Init(print,FL_Options,FL_window,"FL_Options")

-- Help text
help = {
	pre = "fl",
	arg = {
		[""] = "Display fishing details.",
		[" "] = "Display fishing proficiency.",
		["<link>"] = "Display item details.",
		["catch"] = "List fishing catch records.",
		["cook"] = "List fish that can be cooked.",
		["trophy"] = "List fish that are trophies.",
	},
	cmd = {
		fll = {
			[" "] = "Fishing location commands.",
			[";loc"] = "Set the current fishing location.",
			["<loc>"] = "List catches at <loc> fishing location.",
			["last"] = "List catches at last fishing location.",
			["list"] = "List catch count by fishing location.",
		},
		flw = "Open Fishing Log window.",
	},
}