-- Fishing Log window handler
-- coding: utf-8 '�

import "Turbine.UI.Lotro"
import "Vinny.Common"

local labelFont = Turbine.UI.Lotro.Font.Verdana14
local foreColor = Turbine.UI.Color( 0.9, 0.9, 0 )
local whiteColor = Turbine.UI.Color( 1.0, 1.0, 1.0 )
local backColor = Turbine.UI.Color( 0.0, 0.0, 0.0 )
local greyColor = Turbine.UI.Color( 0.1, 0.1, 0.1 )
local Button = Turbine.UI.Lotro.GoldButton
local Label = Turbine.UI.Label
local TextBox = Turbine.UI.TextBox
local CheckBox = Turbine.UI.Lotro.CheckBox
local Item = Turbine.UI.Lotro.ShortcutType.Item
local Hobby = Turbine.UI.Lotro.ShortcutType.Hobby
local Alias = Turbine.UI.Lotro.ShortcutType.Alias
local Shortcut = Turbine.UI.Lotro.Shortcut
local Quickslot = Turbine.UI.Lotro.Quickslot
local Qsize = 34
local Blank

FL_Window = class( Turbine.UI.Lotro.Window )

function FL_Window:AddField(control, text, pos, size)
	local field = control()
	field:SetParent( self )
	if text then field:SetText( text ) end
	field:SetPosition( pos.x,pos.y )
	field:SetSize( size.x,size.y )
	if control==Button or not text then return field end
	field:SetFont( labelFont )
	field:SetForeColor( text=="" and whiteColor or foreColor )
	local grey = control==TextBox or control==Quickslot
	field:SetBackColor( grey and greyColor or backColor )
	field:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft )
	return field
end

function FL_Shortcut(sender,name,iname)
	local shortcut = sender:GetShortcut()
	local itemType = shortcut:GetType()
	if itemType==0 then return end
	local itemData = shortcut:GetData()
	if sender:IsAltKeyDown() then print("Type="..itemType..", Data="..itemData) end
	if itemType~=Item then 
		sender:SetShortcut(Blank) 
		print(name.." reset.")
		return 
	end
	local Item = shortcut:GetItem()
	if not Item then printe("Item is null.") return end
	if iname and Item:GetName():sub(-#iname)~=iname then
		printe(Item:GetName().." is not a "..iname)
		sender:SetShortcut(Blank)
		return
	end
	print(name.." set to "..Item:GetName())
	return itemData
end

function FL_Window:Constructor()
	Turbine.UI.Lotro.Window.Constructor( self )

	-- Position the window near the top center of the screen.
	self:SetSize( 340,220 )
--	self:SetBackColor( Turbine.UI.Color() )
	self:SetPosition( (Turbine.UI.Display.GetWidth() - self:GetWidth())/3, 
						(Turbine.UI.Display:GetHeight() - self:GetHeight())*.6 )
	self:SetText( "Fishing Log" )
	self:SetVisible( false )

-- Hobby:Fishing action is Type=Hobby(9), Data=0x7000EE1E

	-- Create a Name field
	self.name = self:AddField(Label, "Fishing rod:", {x=30,y=47}, {x=80,y=16} )
	self.name:SetFont(Turbine.UI.Lotro.Font.TrajanPro18)

	-- Create an rod field
	self.rod = self:AddField(Quickslot, nil, {x=115,y=40}, {x=Qsize,y=Qsize} )
	Blank = self.rod:GetShortcut()
	if Totals.rod then self.rod:SetShortcut( Shortcut(Item,Totals.rod) ) 
	else self.rod:SetBackground("Vinny/FishingLog/Rod.tga") end
	self.rod.ShortcutChanged = function( sender, args )
		Totals.rod = FL_Shortcut(sender,"Fishing rod","Fishing Rod")
	end

	-- Create a fishing label
	self:AddField(Label, "Fish:", {x=180,y=45}, {x=30,y=16} )

	-- Create an fishing field
	self.fish = self:AddField(Quickslot, nil, {x=220,y=40}, {x=Qsize,y=Qsize} )
	self.fish:SetShortcut( Shortcut(Hobby,"0x7000EE1E") )
    self.fish:SetAllowDrop( false )
	self.fish.MouseEnter = function( sender, args ) Track = true end
	self.fish.MouseLeave = function( sender, args ) Track = false end

	-- Create a weapon label
	self:AddField(Label, "Weapon:", {x=45,y=95}, {x=70,y=16} )

	-- Create an weapon field, weapon slot=16, cat=104
	self.weapon = self:AddField(Quickslot, nil, {x=115,y=90}, {x=Qsize,y=Qsize} )
	if Totals.wpn then self.weapon:SetShortcut( Shortcut(Item,Totals.wpn) ) 
	else self.weapon:SetBackground("Vinny/FishingLog/Sword.tga") end
	self.weapon.ShortcutChanged = function( sender, args )
		Totals.wpn = FL_Shortcut(sender,"Weapon")
	end

	-- Create a Shield label
	self:AddField(Label, "2nd:", {x=180,y=95}, {x=30,y=16} )

	-- Create an shield field, shield slot=17
	self.shield = self:AddField(Quickslot, nil, {x=220,y=90}, {x=Qsize,y=Qsize} )
	if Totals.shl then self.shield:SetShortcut( Shortcut(Item,Totals.shl) ) 
	else self.shield:SetBackground("Vinny/FishingLog/Shield.tga") end
	self.shield.ShortcutChanged = function( sender, args )
		Totals.shl = FL_Shortcut(sender,"2nd")
	end

	-- Create a set location button
	self.locButton = self:AddField(Button, "Set Location", {x=30,y=140}, {x=125,y=20} )
	local slot = Turbine.UI.Lotro.Quickslot()
	slot:SetParent( self.locButton )
    slot:SetPosition( 2,2 )
    slot:SetSize( 99, 15 )
    slot:SetShortcut(Turbine.UI.Lotro.Shortcut( Alias,"/fll ;loc" ))
    slot:SetAllowDrop( false )

	-- Create a Inventory listing button
	self.listButton = self:AddField(Button, "List Locations", {x=175,y=140}, {x=135,y=20} )
	self.listButton.Click = function( sender,args )
        FL_Command:Execute("fll","list")
	end

	-- Create a catch listing button
	self.catchButton = self:AddField(Button, "Loc. Catches", {x=30,y=170}, {x=125,y=20} )
	self.catchButton.Click = function( sender,args )
        FL_Command:Execute("fll","last")
	end

	-- Create a catch listing button
	self.catchButton = self:AddField(Button, "Personal Catches", {x=175,y=170}, {x=135,y=20} )
	self.catchButton.Click = function( sender,args )
        FL_Command:Execute("fl","catch")
	end

end

function FL_Window:SetItem(id,item,xname)
    if not item then item = Mats[id] end
    if not xname then xname = string.format(xlink,id,item.N) end
	self.id = id
	self.item = item
	self.name:SetText( item.N )
	self.tier:SetText(item.T.." ("..Tier[item.T]..")")
	local desc = item.P and Proc[item.P] or "Guild rep item"
	self.type:SetText( desc )
end

FL_window = FL_Window()

-- Set Escape action
FL_window:SetWantsKeyEvents( true )
FL_window.KeyDown = function(sender, args)
	if( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		FL_window:SetVisible( false )
	end
end
